/*
 * Copyright (c) 1999, Boris Popov
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Boris Popov.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: pam_netware.c,v 1.1.2.4 1999/09/30 10:31:00 bp Exp $
 */
#include <sys/types.h>
#include <pwd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#define PAM_SM_AUTH
/*#define PAM_STATIC*/
#include <security/pam_modules.h>
#include <security/pam_mod_misc.h>

#include <netncp/ncp_lib.h>

#define PASSWORD_PROMPT	"Password:"

struct ncp_authopt {
	const char *	opt_name;
	const char *	opt_value;
};

enum ncp_opts {OPT_SERVER, OPT_TREE, OPT_IPADDR, OPT_NONE};

static struct ncp_authopt ncp_optlist[OPT_NONE + 1] = {
	{"server=",	NULL},
	{"tree=",	NULL},
	{"ipaddr=",	NULL},
	{NULL,		NULL}
};

static int 
ncp_auth(const char *server, const char *user, const char *password) {
	struct ncp_conn_args li;
	int error;
	NWCONN_HANDLE connid;
	const char *ipaddr;

	if (ncp_initlib())
		return PAM_AUTH_ERR;;
	if (ncp_li_init(&li, 0, NULL))
		return PAM_AUTH_ERR;
	li.access_mode = 0700;
	
	ncp_li_setserver(&li, server);

	ipaddr = ncp_optlist[OPT_IPADDR].opt_value;
	error = ncp_find_fileserver(&li, ipaddr ? AF_INET : AF_IPX, (char*)ipaddr);
	if (error) {
		syslog(LOG_CRIT, "Couldn't find server %s\n", server);
		return PAM_AUTH_ERR;
	}
	error = ncp_connect(&li, &connid);
	if (!error)
		error = ncp_login(connid, user, li.objtype, password);
	ncp_li_done(&li);
	if (error == 0) {
		ncp_disconnect(connid);
	} else {
/*		pam_prompt(pamh, PAM_ERROR_MSG, "can't authenticate", NULL);*/
	}
        return error ? PAM_AUTH_ERR : PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv) {
	int error, options, i, len;
	const char *user;
	const char *password;
	const char *server;
	struct ncp_authopt *op;

	server = NULL;
	options = 0;
	for (i = 0; i < argc;  i++) {
		pam_std_option(&options, argv[i]);
		for (op = ncp_optlist; op->opt_name; op++) {
			len = strlen(op->opt_name);
			if (strncmp(op->opt_name, argv[i], len) == 0) {
				op->opt_value = argv[i] + len;
				break;
			}
		}
	}
	error = pam_get_user(pamh, &user, NULL);
	if (error != PAM_SUCCESS)
		return error;
	error = pam_get_pass(pamh, &password, PASSWORD_PROMPT, options);
	if (error != PAM_SUCCESS)
		return error;

	server = ncp_optlist[OPT_SERVER].opt_value;
	if (server == NULL || server[0] == 0)
	        return PAM_AUTH_ERR;

	return ncp_auth(server, user, password);
}

PAM_EXTERN int
pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, const char **argv) {
	return PAM_IGNORE;
}


PAM_MODULE_ENTRY("pam_netware");

