/*
 *  Change password for a bindery object
 *
 *  Copyright 1999, Boris Popov
 *
 *  $Id: ncpasswd.c,v 1.1.2.3 1999/09/30 10:31:01 bp Exp $
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netncp/ncp_lib.h>
#include <netncp/ncp_rcfile.h>

extern char *__progname;

static void
usage(void) {
	fprintf(stderr, "usage: %s [options]\n", __progname);
}

static void
help(void)
{
	printf("\n");
	printf("usage: %s [options] [objectname]\n", __progname);
	printf("\n"
	       "-h             Print this help text\n"
	       "-S server      Server name to be used\n"
	       "-U username    Username sent to server\n"
	       "-t type        Object type (decimal value)\n"
	       "-v             Verify an entered password\n"
	       "\n");
}

static int
ncp_getpass(char *prompt, char *buf) {
	char *str;

	str = getpass(prompt);
	if (strlen(str) >= NCP_MAXPASSWORDLEN) {
		printf("Password too long (%d symbols max)\n", NCP_MAXPASSWORDLEN);
		exit(1);
	}
	ncp_str_upper(strcpy(buf, str));
	return 0;
}

int
main(int argc, char *argv[]) {
	NWCONN_HANDLE connid;
	struct ncp_conn_loginfo li;
	char *obj_name = NULL;
	int object_type = NCP_BINDERY_USER;
	int error, opt, diffobj = 0, doverify = 0, is_super;
	unsigned char ncp_key[8];
	struct ncp_bindery_object objinfo;
	char oldpass[NCP_MAXPASSWORDLEN];
	char newpass1[NCP_MAXPASSWORDLEN], newpass2[NCP_MAXPASSWORDLEN];

	if (ncp_initlib()) exit(1);
	if (ncp_li_init(&li, argc, argv)) return 1;
	if (ncp_li_readrc(&li)) return 1;
	if (ncp_rc)
		rc_close(ncp_rc);

	while ((opt = getopt(argc, argv, STDPARAM_OPT"h?S:U:O:t:v")) != EOF) {
		switch (opt) {
		    case STDPARAM_ARGS:
			if (ncp_li_arg(&li, opt, optarg)) {	
				return 1;
			}
			break;
		    case 't':
			object_type = atoi(optarg);
			break;
		    case 'v':
			doverify = 1;
			break;
		    case 'h':
			help();
			exit(1);
		    default:
			usage();
			exit(1);
		}
	}
	if (ncp_li_check(&li)) return 1;
	li.opt |= NCP_OPT_WDOG;
	error = ncp_li_login(&li,&connid);
	if (error) {
		ncp_error("Can't find suitable connection", error);
		exit(1);
	}

	if ((argc - optind) == 1) {
		obj_name = argv[optind];
		diffobj = 1;
	} else {
		obj_name = li.user;
	}
	li.objtype = object_type;

	error =  ncp_get_bindery_object_id(connid, 1, li.user, &objinfo);
	if (error) {
		ncp_error("Can't get object info", error);
		exit(1);
	}
	is_super = objinfo.object_id == 1;

	printf("%s password for user %s on server %s\n",
	    doverify ? "Verifying" : "Changing", obj_name, li.server);

	error = ncp_get_encryption_key(connid, ncp_key);
	if (error) {
		ncp_error("Can't get encryption key", error);
		return 1;
	}
	error =  ncp_get_bindery_object_id(connid, 1, obj_name, &objinfo);
	if (error) {
		ncp_error("Can't get object info", error);
		exit(1);
	}

	if (!is_super || doverify) {
		if (!diffobj) {
			ncp_getpass("Enter old password: ", oldpass);
		} else {
			char temp[80];
			sprintf(temp, "Enter password for %s: ", obj_name);
			ncp_getpass(temp, oldpass);
		}
		error = ncp_keyed_verify_password(connid, ncp_key, oldpass, &objinfo);
		if (error) {
			ncp_error("Wrong old password", error);
			exit(1);
		}
		if (doverify) {
			printf("Password is ok.\n");
			return 0;
		}
	} else {
		oldpass[0] = 0;
	}
	ncp_getpass("Enter new password: ", newpass1);
	ncp_getpass("Re-Enter new password: ", newpass2);

	if (strcmp(newpass1, newpass2) != 0) {
		printf("Mismatch; try again\n");
		exit(1);
	}
	error = ncp_change_obj_passwd(connid, &objinfo, ncp_key, oldpass, newpass1);
	if (error) {
		ncp_error("Can't change password",error);
		return 1;
	}
	return 0;
}
